@@ -200,8 +200,9 @@ gregorio_set_signs (gregorio_note *note, char signs)
 void
 gregorio_change_shape (gregorio_note *note, char shape)
 {
-    if (!note) {
-      // error
+    if (!note || note->type != GRE_NOTE) {
+      gregorio_message (_("trying to change the shape of something that is not a note"),
+			   "change_shape", ERROR, 0);
       return;
     }
     note->shape=shape;
@@ -241,8 +242,9 @@ gregorio_change_shape (gregorio_note *note, char shape)
 void
 gregorio_add_liquescentia (gregorio_note *note, char liq)
 {
-    if (!note) {
-      // error
+    if (!note || note->type != GRE_NOTE) {
+      gregorio_message (_("trying to make a liquescence on something that is not a note"),
+			   "add_liquescentia", ERROR, 0);
       return;
     }
     if (is_initio_debilis(liq))
@@ -303,12 +305,24 @@ gregorio_add_liquescentia (gregorio_note *note, char liq)
 void
 gregorio_add_h_episemus (gregorio_note *note, unsigned char type, unsigned int *nbof_isolated_episemus) 
 {
+  if (!note || (note->type != GRE_NOTE && note->type != GRE_BAR))
+    {
+      gregorio_message (_("trying to add an horizontal episemus on something that is not a note"),
+			   "add_h_episemus", ERROR, 0);
+      return;
+    }
+  if (!nbof_isolated_episemus)
+    {
+      gregorio_message (_("NULL argument nbof_isolated_episemus"),
+			   "add_h_episemus", FATAL_ERROR, 0);
+      return;
+    }
   if (type == H_BOTTOM)
     {
       note->h_episemus_type = note->h_episemus_type | H_BOTTOM;
       return;
     }
-  if (!note->h_episemus_top_note || nbof_isolated_episemus == NULL || *nbof_isolated_episemus == 0)
+  if (!note->h_episemus_top_note || *nbof_isolated_episemus == 0)
     {
       gregorio_mix_h_episemus (note, H_ONE);
       *nbof_isolated_episemus = 1;
@@ -324,6 +338,12 @@ gregorio_add_h_episemus (gregorio_note *note, unsigned char type, unsigned int *
 void
 gregorio_set_h_episemus(gregorio_note *note, unsigned char type)
 {
+  if (!note || (note->type != GRE_NOTE && note->type != GRE_BAR))
+    {
+      gregorio_message (_("trying to add an horizontal episemus on something that is not a note"),
+			   "set_h_episemus", ERROR, 0);
+      return;
+    }
   if (type == H_BOTTOM)
     {
       note->h_episemus_type = (note->h_episemus_type) | H_BOTTOM;
@@ -1392,7 +1412,7 @@ gregorio_set_voice_virgula_position (gregorio_voice_info * voice_info,
 /**********************************
  *
  * Activate_isolated_h_episemus is used when we see an "isolated"
- * horizontal episemus: when we type ab__ lex see a then b_ then _, so
+ * horizontal episemus: when we type ab__ lex see a then b then _ then _, so
  * we must put the _ on the a (kind of backward process), and say the
  * the episemus on the b is a multi episemus. Here n is the length of
  * the isolated episemus we found (can be up to 4).
@@ -1431,12 +1451,14 @@ gregorio_activate_isolated_h_episemus (gregorio_note * current_note, int n)
 			   "activate_h_isolated_episemus", WARNING, 0);
       return;
     }
-  for (i = 0; i < n; i++)
+  top_note = max (top_note, tmp->pitch);
+  for (i = 0; i < n-1; i++)
     {
       top_note = max (top_note, tmp->pitch);
       if (tmp->previous && tmp->previous->type == GRE_NOTE)
 	{
 	  tmp = tmp->previous;
+	  top_note = max (top_note, tmp->pitch);
 	}
       else
 	{
@@ -1446,6 +1468,13 @@ gregorio_activate_isolated_h_episemus (gregorio_note * current_note, int n)
 	  break;
 	}
     }
+  // improvement: we consider also the previous note (if it's a GRE_NOTE) for
+  // the top note. TODO: we should consider also the next note, but we cannot
+  // do it at this stage.
+  if (tmp->previous && tmp->previous->type == GRE_NOTE)
+    {
+      top_note = max (top_note, tmp->previous->pitch);
+    }
   while (tmp)
     {
       gregorio_set_h_episemus(tmp, H_MULTI);
@@ -1533,7 +1562,14 @@ gregorio_mix_h_episemus (gregorio_note * current_note, unsigned char type)
     }
   else
     {
-      current_note->h_episemus_top_note = current_note->pitch;
+      if (prev_note && prev_note->type == GRE_NOTE)
+        {
+          current_note->h_episemus_top_note = max(prev_note->pitch, current_note->pitch);
+        }
+      else
+        {
+          current_note->h_episemus_top_note = current_note->pitch;
+        }
       if (!prev_note || prev_note->type != GRE_NOTE
 	  || simple_htype(prev_note->h_episemus_type) == H_NO_EPISEMUS)
 	{
