@@ -1,6 +1,6 @@
-/* 
+/*
 Gregorio xml output format.
-Copyright (C) 2006 Elie Roux <elie.roux@enst-bretagne.fr>
+Copyright (C) 2006-2009 Elie Roux <elie.roux@telecom-bretagne.eu>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,20 +19,34 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
-#include "gettext.h"
-#define _(str) gettext(str)
-#define N_(str) str
 #include <gregorio/struct.h>
 #include <gregorio/unicode.h>
 #include <gregorio/characters.h>
 #include <gregorio/messages.h>
+#if ALL_STATIC == 0
+    #include <gregorio/plugin.h>
+#endif
 
 #include "xml.h"
 
-#define write_note(shape) libgregorio_xml_write_note(f, note->signs, step, octave, shape, note->h_episemus_type, alteration, note->rare_sign)
+#if ALL_STATIC == 0
+DECLARE_PLUGIN (xml)
+{
+.id = "xml",.name = "GregorioXML",.description =
+    "GregorioXML input/output plugin",.author =
+    "Elie Roux <elie.roux@enst-bretagne.fr>",.file_extension = "xml",
+    #if ENABLE_XML == 1
+    .type =GREGORIO_PLUGIN_BOTH,.read = read_score,
+    #else
+    .type =GREGORIO_PLUGIN_OUTPUT,
+    #endif
+    .write = write_score};
+#endif
+
+#define write_note(shape) xml_write_note(f, note->signs, step, octave, shape, note->h_episemus_type, alteration, note->rare_sign, note->texverb)
 
 void
-libgregorio_xml_write_gregorio_note (FILE * f, gregorio_note * note, int clef,
+xml_write_gregorio_note (FILE * f, gregorio_note * note, int clef,
 				     char alterations[])
 {
   char step;
@@ -42,21 +56,21 @@ libgregorio_xml_write_gregorio_note (FILE * f, gregorio_note * note, int clef,
   if (!f) {
       gregorio_message (_
 			   ("call with NULL file"),
-			   "libgregorio_gregoriotex_write_score", ERROR, 0);
+			   "gregoriotex_write_score", ERROR, 0);
 			   return;
   }
 
   if (!note)
     {
       gregorio_message (_("call with NULL argument"),
-			   "libgregorio_xml_write_gregorio_note", ERROR, 0);
+			   "xml_write_gregorio_note", ERROR, 0);
       return;
     }
   if (note->type != GRE_NOTE)
     {
       gregorio_message (_
 			   ("call with argument which type is not GRE_NOTE, wrote nothing"),
-			   "libgregorio_xml_write_gregorio_note", ERROR, 0);
+			   "xml_write_gregorio_note", ERROR, 0);
       return;
     }
   gregorio_determine_h_episemus_type (note);
@@ -95,7 +109,7 @@ libgregorio_xml_write_gregorio_note (FILE * f, gregorio_note * note, int clef,
 }
 
 void
-libgregorio_xml_write_gregorio_glyph (FILE * f, gregorio_glyph * glyph,
+xml_write_gregorio_glyph (FILE * f, gregorio_glyph * glyph,
 				      int clef, char alterations[])
 {
   const char *type;
@@ -104,12 +118,12 @@ libgregorio_xml_write_gregorio_glyph (FILE * f, gregorio_glyph * glyph,
   if (!glyph)
     {
       gregorio_message (_("call with NULL argument"),
-			   "libgregorio_xml_write_gregorio_glyph", ERROR, 0);
+			   "xml_write_gregorio_glyph", ERROR, 0);
       return;
     }
   if (is_alteration (glyph->type))
     {
-      libgregorio_xml_write_alteration (f, glyph->type, glyph->glyph_type,
+      xml_write_alteration (f, glyph->type, glyph->glyph_type,
 					clef, alterations);
       return;
     }
@@ -118,27 +132,35 @@ libgregorio_xml_write_gregorio_glyph (FILE * f, gregorio_glyph * glyph,
       fprintf (f, "<zero-width-space />");
       return;
     }
+  if (glyph->type == GRE_TEXVERB_GLYPH)
+    {
+      if (glyph->texverb)
+        {
+          fprintf(f, "<texverb-glyph>%s</texverb-glyph>", glyph->texverb);
+        }
+      return;
+    }
   if (glyph->type != GRE_GLYPH)
     {
       gregorio_message (_("call with an argument which type is unknown"),
-			   "libgregorio_xml_write_gregorio_glyph", ERROR, 0);
+			   "xml_write_gregorio_glyph", ERROR, 0);
       return;
     }
-  type = libgregorio_xml_glyph_type_to_str (glyph->glyph_type);
+  type = xml_glyph_type_to_str (glyph->glyph_type);
   fprintf (f, "<glyph><type>%s</type>", type);
-  libgregorio_xml_write_liquescentia (f, glyph->liquescentia);
+  xml_write_liquescentia (f, glyph->liquescentia);
   current_note = glyph->first_note;
   while (current_note)
     {
-      libgregorio_xml_write_gregorio_note (f, current_note, clef,
+      xml_write_gregorio_note (f, current_note, clef,
 					   alterations);
-      current_note = current_note->next_note;
+      current_note = current_note->next;
     }
   fprintf (f, "</glyph>");
 }
 
 void
-libgregorio_xml_write_gregorio_element (FILE * f, gregorio_element * element,
+xml_write_gregorio_element (FILE * f, gregorio_element * element,
 					int *clef, char alterations[])
 {
   gregorio_glyph *current_glyph;
@@ -146,7 +168,7 @@ libgregorio_xml_write_gregorio_element (FILE * f, gregorio_element * element,
   if (!element)
     {
       gregorio_message (_("call with NULL argument"),
-			   "libgregorio_xml_write_gregorio_element", ERROR,
+			   "xml_write_gregorio_element", ERROR,
 			   0);
       return;
     }
@@ -156,28 +178,38 @@ libgregorio_xml_write_gregorio_element (FILE * f, gregorio_element * element,
       current_glyph = element->first_glyph;
       while (current_glyph)
 	{
-	  libgregorio_xml_write_gregorio_glyph (f, current_glyph, *clef,
+	  xml_write_gregorio_glyph (f, current_glyph, *clef,
 						alterations);
-	  current_glyph = current_glyph->next_glyph;
+	  current_glyph = current_glyph->next;
 	}
       fprintf (f, "</element>");
       return;
     }
   if (element->type == GRE_SPACE)
     {
-      libgregorio_xml_write_space (f, element->element_type);
+      xml_write_space (f, element->element_type);
+      return;
+    }
+  if (element->type == GRE_TEXVERB_ELEMENT)
+    {
+      if (element->texverb)
+        {
+          fprintf(f, "<texverb-element>%s</texverb-element>", element->texverb);
+        }
       return;
     }
   if (element->type == GRE_BAR)
     {
       gregorio_reinitialize_one_voice_alterations (alterations);
-      libgregorio_xml_write_neumatic_bar (f, element->element_type);
+      fprintf (f, "<neumatic-bar>");
+      xml_write_bar (f, element->element_type, element->additional_infos);
+      fprintf (f, "</neumatic-bar>");
       return;
     }
   if (element->type == GRE_CUSTO)
     {
       fprintf(f, "<custo>");
-      libgregorio_xml_write_pitch (f, element->element_type, *clef);
+      xml_write_pitch (f, element->element_type, *clef);
       fprintf(f, "</custo>");
       return;
     }
@@ -190,24 +222,24 @@ libgregorio_xml_write_gregorio_element (FILE * f, gregorio_element * element,
     {
       *clef =
 	gregorio_calculate_new_key (C_KEY, element->element_type - 48);
-      libgregorio_xml_write_key_change (f, C_KEY, element->element_type - 48);
+      xml_write_key_change (f, C_KEY, element->element_type - 48);
       return;
     }
   if (element->type == GRE_F_KEY_CHANGE)
     {
       *clef =
 	gregorio_calculate_new_key (F_KEY, element->element_type - 48);
-      libgregorio_xml_write_key_change (f, F_KEY, element->element_type - 48);
+      xml_write_key_change (f, F_KEY, element->element_type - 48);
       return;
     }
 
   gregorio_message (_("call with an argument which type is unknown"),
-		       "libgregorio_xml_write_gregorio_element", ERROR, 0);
+		       "xml_write_gregorio_element", ERROR, 0);
 
 }
 
 void
-libgregorio_xml_write_space (FILE * f, char type)
+xml_write_space (FILE * f, char type)
 {
   switch (type)
     {
@@ -217,26 +249,49 @@ libgregorio_xml_write_space (FILE * f, char type)
     case SP_GLYPH_SPACE:
       fprintf (f, "<glyph-space />");
       break;
+    case SP_GLYPH_SPACE_NB:
+      fprintf (f, "<unbreakable-glyph-space />");
+      break;
+    case SP_LARGER_SPACE_NB:
+      fprintf (f, "<unbreakable-larger-neumatic-space />");
+      break;
+    case SP_NEUMATIC_CUT_NB:
+      fprintf (f, "<unbreakable-neumatic-cut />");
+      break;
     case SP_NEUMATIC_CUT:
       //we do not write neumatic cuts, they juste delimit elements
       break;
     default:
       gregorio_message (_("space type is unknown"),
-			   "libgregorio_xml_write_space", ERROR, 0);
+			   "xml_write_space", ERROR, 0);
       break;
     }
 }
 
 void
-libgregorio_xml_write_neumatic_bar (FILE * f, char type)
+xml_write_bar (FILE * f, char type, char signs)
 {
   const char *str;
-  str = libgregorio_xml_bar_to_str (type);
-  fprintf (f, "<neumatic-bar><type>%s</type></neumatic-bar>", str);
+  str = xml_bar_to_str (type);
+  fprintf (f, "<type>%s</type>", str);
+  switch (signs)
+    {
+    case _V_EPISEMUS:
+      fprintf(f, "<signs>vertical-episemus</signs>");
+      break;
+    case _V_EPISEMUS_ICTUS_A:
+      fprintf(f, "<signs>vertical-episemus-ictus-a</signs>");
+      break;
+    case _V_EPISEMUS_ICTUS_T:
+      fprintf(f, "<signs>vertical-episemus-ictus-t</signs>");
+      break;
+    default:
+      break;
+    }
 }
 
 void
-libgregorio_xml_write_key_change (FILE * f, char step, int line)
+xml_write_key_change (FILE * f, char step, int line)
 {
   fprintf (f,
 	   "<clef-change><step>%c</step><line>%d</line></clef-change>",
@@ -244,7 +299,7 @@ libgregorio_xml_write_key_change (FILE * f, char step, int line)
 }
 
 void
-libgregorio_xml_write_key_change_in_polyphony (FILE * f, char step, int line,
+xml_write_key_change_in_polyphony (FILE * f, char step, int line,
 					       int voice)
 {
   fprintf (f,
@@ -255,7 +310,7 @@ libgregorio_xml_write_key_change_in_polyphony (FILE * f, char step, int line,
 char in_text = 0;
 
 void
-libgregorio_xml_write_begin (FILE * f, unsigned char style)
+xml_write_begin (FILE * f, unsigned char style)
 {
   if (in_text)
     {
@@ -283,13 +338,16 @@ libgregorio_xml_write_begin (FILE * f, unsigned char style)
     case ST_TT:
       fprintf (f, "<tt>");
       break;
+    case ST_UNDERLINED:
+      fprintf (f, "<ul>");
+      break;
     default:
       break;
     }
 }
 
 void
-libgregorio_xml_write_end (FILE * f, unsigned char style)
+xml_write_end (FILE * f, unsigned char style)
 {
   if (in_text)
     {
@@ -317,13 +375,16 @@ libgregorio_xml_write_end (FILE * f, unsigned char style)
     case ST_TT:
       fprintf (f, "</tt>");
       break;
+    case ST_UNDERLINED:
+      fprintf (f, "</ul>");
+      break;
     default:
       break;
     }
 }
 
 void
-libgregorio_xml_write_special_char (FILE * f, grewchar *first_char)
+xml_write_special_char (FILE * f, grewchar *first_char)
 {
   if (in_text)
     {
@@ -331,16 +392,12 @@ libgregorio_xml_write_special_char (FILE * f, grewchar *first_char)
       in_text = 0;
     }
   fprintf (f, "<special-char>");
-  while (*first_char != 0)
-    {
-      fprintf (f, "%lc", *first_char);
-      first_char ++;
-    }
+  xml_print_unistring(f, first_char);
   fprintf (f, "</special-char>");
 }
 
 void
-libgregorio_xml_write_verb (FILE * f, grewchar *first_char)
+xml_write_verb (FILE * f, grewchar *first_char)
 {
   if (in_text)
     {
@@ -348,27 +405,62 @@ libgregorio_xml_write_verb (FILE * f, grewchar *first_char)
       in_text = 0;
     }
   fprintf (f, "<verbatim>");
-  while (*first_char != 0)
-    {
-      fprintf (f, "%lc", *first_char);
-      first_char ++;
-    }
+  xml_print_unistring(f, first_char);
   fprintf (f, "</verbatim>");
 }
 
 void
-libgregorio_xml_print_char (FILE * f, grewchar to_print)
+xml_print_unistring(FILE *f, grewchar *first_char)
+{
+  while (*first_char != 0)
+    {
+      xml_print_unichar (f, *first_char);
+      first_char++;
+    }
+}
+
+void
+xml_print_unichar (FILE * f, grewchar to_print)
+{
+  switch (to_print)
+    {
+    case L'<':
+      fprintf (f, "&lt;");
+      break;
+    case L'>':
+      fprintf (f, "&gt;");
+      break;
+    case L'\'':
+      fprintf (f, "&apos;");
+      break;
+    case L'&':
+      fprintf (f, "&amp;");
+      break;
+    case L'"':
+      fprintf (f, "\\\"");
+      break;
+    case L'\\':
+      fprintf (f, "\\\\");
+      break;
+   default:
+     gregorio_print_unichar(f, to_print);
+     break;
+   }
+}
+
+void
+xml_print_char (FILE * f, grewchar to_print)
 {
   if (!in_text)
     {
       fprintf (f, "<str>");
       in_text = 1;
     }
-  fprintf (f, "%lc", to_print);
+   xml_print_unichar(f, to_print);
 }
 
 void
-libgregorio_xml_print_text (FILE * f, gregorio_character * text,
+xml_print_text (FILE * f, gregorio_character * text,
 			    char position)
 {
   const char *position_str;
@@ -396,11 +488,11 @@ libgregorio_xml_print_text (FILE * f, gregorio_character * text,
     }
   fprintf (f, "<text position=\"%s\">", position_str);
   gregorio_write_text (0, text, f,
-			  (&libgregorio_xml_write_verb),
-			  (&libgregorio_xml_print_char),
-			  (&libgregorio_xml_write_begin),
-			  (&libgregorio_xml_write_end),
-			  (&libgregorio_xml_write_special_char));
+			  (&xml_write_verb),
+			  (&xml_print_char),
+			  (&xml_write_begin),
+			  (&xml_write_end),
+			  (&xml_write_special_char));
   if (in_text)
     {
       fprintf (f, "</str>");
@@ -410,7 +502,7 @@ libgregorio_xml_print_text (FILE * f, gregorio_character * text,
 }
 
 void
-libgregorio_xml_print_translation (FILE * f, gregorio_character * translation)
+xml_print_translation (FILE * f, gregorio_character * translation)
 {
   if (!translation)
     {
@@ -418,11 +510,11 @@ libgregorio_xml_print_translation (FILE * f, gregorio_character * translation)
     }
   fprintf (f, "<translation>");
   gregorio_write_text (0, translation, f,
-			  (&libgregorio_xml_write_verb),
-			  (&libgregorio_xml_print_char),
-			  (&libgregorio_xml_write_begin),
-			  (&libgregorio_xml_write_end),
-			  (&libgregorio_xml_write_special_char));
+			  (&xml_write_verb),
+			  (&xml_print_char),
+			  (&xml_write_begin),
+			  (&xml_write_end),
+			  (&xml_write_special_char));
   if (in_text)
     {
       fprintf (f, "</str>");
@@ -433,7 +525,7 @@ libgregorio_xml_print_translation (FILE * f, gregorio_character * translation)
 
 
 void
-libgregorio_xml_write_syllable (FILE * f, gregorio_syllable * syllable,
+xml_write_syllable (FILE * f, gregorio_syllable * syllable,
 				int number_of_voices, int clefs[],
 				char alterations[][13])
 {
@@ -443,7 +535,7 @@ libgregorio_xml_write_syllable (FILE * f, gregorio_syllable * syllable,
   if (!syllable)
     {
       gregorio_message (_("call with NULL argument"),
-			   "libgregorio_xml_write_syllable", ERROR, 0);
+			   "xml_write_syllable", ERROR, 0);
     }
   if (syllable->position == WORD_BEGINNING)
     {
@@ -460,29 +552,29 @@ libgregorio_xml_write_syllable (FILE * f, gregorio_syllable * syllable,
   fprintf (f, "<syllable>");
   if (syllable->text)
     {
-      libgregorio_xml_print_text (f, syllable->text, syllable->position);
+      xml_print_text (f, syllable->text, syllable->position);
     }
   if (syllable->translation)
     {
-      libgregorio_xml_print_translation (f, syllable->translation);
+      xml_print_translation (f, syllable->translation);
     }
   for (i = 0; i < number_of_voices; i++)
     {
       if (!(syllable->elements[i]))
 	{
 	  gregorio_message (_("not_enough voices in syllable"),
-			       "libgregorio_xml_write_syllable", VERBOSE, 0);
+			       "xml_write_syllable", VERBOSE, 0);
 	  voice++;
 	  continue;
 	}
       if (gregorio_is_only_special (syllable->elements[i]))
 	{
-	  libgregorio_xml_write_specials_as_neumes (f, syllable->elements[i],
+	  xml_write_specials_as_neumes (f, syllable->elements[i],
 						    voice, &clefs[i]);
 	}
       else
 	{
-	  libgregorio_xml_write_neume (f, syllable->elements[i], voice,
+	  xml_write_neume (f, syllable->elements[i], voice,
 				       &clefs[i], alterations[i]);
 	}
       voice++;
@@ -493,7 +585,7 @@ libgregorio_xml_write_syllable (FILE * f, gregorio_syllable * syllable,
 }
 
 void
-libgregorio_xml_write_neume (FILE * f, gregorio_element * element, int voice,
+xml_write_neume (FILE * f, gregorio_element * element, int voice,
 			     int *clef, char alterations[])
 {
   if (!element)
@@ -510,37 +602,45 @@ libgregorio_xml_write_neume (FILE * f, gregorio_element * element, int voice,
     }
   while (element)
     {
-      libgregorio_xml_write_gregorio_element (f, element, clef, alterations);
-      element = element->next_element;
+      xml_write_gregorio_element (f, element, clef, alterations);
+      element = element->next;
     }
   fprintf (f, "</neume>");
 }
 
 void
-libgregorio_xml_write_specials_as_neumes (FILE * f,
+xml_write_specials_as_neumes (FILE * f,
 					  gregorio_element * element,
 					  int voice, int *clef)
 {
-  const char *str;
 
   while (element)
     {
       if (element->type == GRE_BAR)
 	{
-	  str = libgregorio_xml_bar_to_str (element->element_type);
 	  if (voice == MONOPHONY)
 	    {
-	      fprintf (f, "<bar><type>%s</type></bar>", str);
+	      fprintf (f, "<bar>");
+          xml_write_bar (f, element->element_type, element->additional_infos);
+          fprintf(f, "</bar>");
 	    }
 	  else
 	    {
-	      fprintf (f, "<bar voice=\"%d\"><type>%s</type></bar>",
-		       voice, str);
+		  fprintf (f, "<bar voice=\"%d\">", voice);
+          xml_write_bar (f, element->element_type, element->additional_infos);
+          fprintf(f, "</bar>");
 	    }
 	}
       if (element->type == GRE_END_OF_LINE)
 	{
+	  if (element->element_type == GRE_END_OF_PAR)
+	    {
+	      fprintf (f, "<end-of-paragraph/>");
+	    }
+	  else
+	    {
 	      fprintf (f, "<end-of-line/>");
+	    }
 	}
       if (element->type == GRE_C_KEY_CHANGE)
 	{
@@ -549,7 +649,7 @@ libgregorio_xml_write_specials_as_neumes (FILE * f,
 	      *clef =
 		gregorio_calculate_new_key (C_KEY,
 					       element->element_type - 48);
-	      libgregorio_xml_write_key_change (f, C_KEY,
+	      xml_write_key_change (f, C_KEY,
 						element->element_type - 48);
 	    }
 	  else
@@ -557,7 +657,7 @@ libgregorio_xml_write_specials_as_neumes (FILE * f,
 	      *clef =
 		gregorio_calculate_new_key (C_KEY,
 					       element->element_type - 48);
-	      libgregorio_xml_write_key_change_in_polyphony (f, C_KEY,
+	      xml_write_key_change_in_polyphony (f, C_KEY,
 							     element->
 							     element_type -
 							     48, voice);
@@ -570,7 +670,7 @@ libgregorio_xml_write_specials_as_neumes (FILE * f,
 	      *clef =
 		gregorio_calculate_new_key (F_KEY,
 					       element->element_type - 48);
-	      libgregorio_xml_write_key_change (f, F_KEY,
+	      xml_write_key_change (f, F_KEY,
 						element->element_type - 48);
 	    }
 	  else
@@ -578,18 +678,23 @@ libgregorio_xml_write_specials_as_neumes (FILE * f,
 	      *clef =
 		gregorio_calculate_new_key (F_KEY,
 					       element->element_type - 48);
-	      libgregorio_xml_write_key_change_in_polyphony (f, F_KEY,
+	      xml_write_key_change_in_polyphony (f, F_KEY,
 							     element->
 							     element_type -
 							     48, voice);
 	    }
 	}
-      element = element->next_element;
+      element = element->next;
     }
 }
 
+#if ALL_STATIC == 0
 void
 write_score (FILE * f, gregorio_score * score)
+#else
+void
+xml_write_score (FILE * f, gregorio_score * score)
+#endif
 {
   int i;
   gregorio_voice_info *voice_info;
@@ -597,7 +702,7 @@ write_score (FILE * f, gregorio_score * score)
   if (!score)
     {
       gregorio_message (_("call with NULL argument"),
-			   "libgregorio_xml_write_score", ERROR, 0);
+			   "xml_write_score", ERROR, 0);
       return;
     }
 
@@ -610,7 +715,7 @@ write_score (FILE * f, gregorio_score * score)
 	{
 	  gregorio_message (_
 			       ("score has more voice infos than voices (attribute number of voices)"),
-			       "libgregorio_xml_write_syllable", ERROR, 0);
+			       "xml_write_syllable", ERROR, 0);
 	  return;
 	}
       clefs[i] = voice_info->initial_key;
@@ -620,13 +725,13 @@ write_score (FILE * f, gregorio_score * score)
 // the clefs array is good, now let's take care of alterations
   char alterations[score->number_of_voices][13];
   gregorio_reinitialize_alterations (alterations, score->number_of_voices);
-
+  fprintf(f, "<?xml version=\"1.0\"?>\n<!-- File generated by gregorio %s -->\n", VERSION);
   fprintf (f, "<score>");
-  libgregorio_xml_write_score_attributes (f, score);
+  xml_write_score_attributes (f, score);
   current_syllable = score->first_syllable;
   while (current_syllable)
     {
-      libgregorio_xml_write_syllable (f, current_syllable,
+      xml_write_syllable (f, current_syllable,
 				      score->number_of_voices, clefs,
 				      alterations);
       current_syllable = current_syllable->next_syllable;
@@ -635,18 +740,19 @@ write_score (FILE * f, gregorio_score * score)
 }
 
 void
-libgregorio_xml_write_score_attributes (FILE * f, gregorio_score * score)
+xml_write_score_attributes (FILE * f, gregorio_score * score)
 {
   //two variables for the key determination
   char step;
   int line;
   int voice = MONOPHONY;
   gregorio_voice_info *current_voice_info;
+  int annotation_num;
 
   if (!score)
     {
       gregorio_message (_("call with NULL argument"),
-			   "libgregorio_xml_write_score_attributes", ERROR,
+			   "xml_write_score_attributes", ERROR,
 			   0);
       return;
     }
@@ -656,7 +762,7 @@ libgregorio_xml_write_score_attributes (FILE * f, gregorio_score * score)
     {
       gregorio_message (_
 			   ("score has no name attribute, which is mandatory"),
-			   "libgregorio_xml_write_score_attributes", ERROR,
+			   "xml_write_score_attributes", ERROR,
 			   0);
       fprintf (f, "<name></name>");
     }
@@ -664,10 +770,34 @@ libgregorio_xml_write_score_attributes (FILE * f, gregorio_score * score)
     {
       fprintf (f, "<name>%s</name>", score->name);
     }
+  if (score->gabc_copyright)
+    {
+      fprintf (f, "<gabc-copyright>%s</gabc-copyright>", score->gabc_copyright);
+    }
+  if (score->score_copyright)
+    {
+      fprintf (f, "<score-copyright>%s</score-copyright>", score->score_copyright);
+    }
   if (score->office_part)
     {
       fprintf (f, "<office-part>%s</office-part>", score->office_part);
     }
+  if (score->occasion)
+    {
+      fprintf (f, "<occasion>%s</occasion>", score->occasion);
+    }
+  if (score->meter)
+    {
+      fprintf (f, "<meter>%s</meter>", score->meter);
+    }
+  if (score->commentary)
+    {
+      fprintf (f, "<commentary>%s</commentary>", score->commentary);
+    }
+  if (score->arranger)
+    {
+      fprintf (f, "<arranger>%s</arranger>", score->arranger);
+    }
   if (score->initial_style)
     {
       fprintf (f, "<initial-style>%d</initial-style>", score->initial_style);
@@ -676,6 +806,50 @@ libgregorio_xml_write_score_attributes (FILE * f, gregorio_score * score)
     {
       fprintf (f, "<mode>%d</mode>", score->mode);
     }
+  if (score->si.author)
+    {
+      fprintf (f, "<author>%s</author>", score->si.author);
+    }
+  if (score->si.date)
+    {
+      fprintf (f, "<date>%s</date>", score->si.date);
+    }
+  if (score->si.manuscript)
+    {
+      fprintf (f,
+	       "<manuscript>%s</manuscript>",
+	       score->si.manuscript);
+    }
+  if (score->si.manuscript_reference)
+    {
+      fprintf (f,
+	       "<manuscript-reference>%s</manuscript-reference>",
+	       score->si.manuscript_reference);
+    }
+  if (score->si.manuscript_storage_place)
+    {
+      fprintf (f,
+	       "<manuscript-storage-place>%s</manuscript-storage-place>",
+	       score->si.manuscript_storage_place);
+    }
+  if (score->si.book)
+    {
+      fprintf (f,
+	       "<book>%s</book>",
+	       score->si.book);
+    }
+  if (score->si.transcriber)
+    {
+      fprintf (f,
+	       "<transcriber>%s</transcriber>",
+	       score->si.transcriber);
+    }
+  if (score->si.transcription_date)
+    {
+      fprintf (f,
+	       "<transcription-date>%s</transcription-date>",
+	       score->si.transcription_date);
+    }
   if (score->lilypond_preamble)
     {
       fprintf (f, "<lilypond-preamble>%s</lilypond-preamble>",
@@ -696,6 +870,11 @@ libgregorio_xml_write_score_attributes (FILE * f, gregorio_score * score)
       fprintf (f, "<gregoriotex_font>%s</gregoriotex_font>",
 	       score->gregoriotex_font);
     }
+  if (score->user_notes)
+    {
+      fprintf (f, "<user_notes>%s</user_notes>",
+	       score->user_notes);
+    }
 
 //then we consider the voice_info
   current_voice_info = score->first_voice_info;
@@ -715,49 +894,14 @@ libgregorio_xml_write_score_attributes (FILE * f, gregorio_score * score)
 	{
 	  fprintf (f, "<voice-info id=\"%d\">", voice);
 	}
-      if (current_voice_info->anotation)
+      for (annotation_num = 0; annotation_num < NUM_ANNOTATIONS; ++annotation_num)
+	{
+	  if (current_voice_info->annotation [annotation_num])
 	{
 	  fprintf (f,
-		   "<anotation>%s</anotation>",
-		   current_voice_info->anotation);
+		       "<annotation>%s</annotation>",
+		       current_voice_info->annotation [annotation_num]);
 	}
-      if (current_voice_info->author)
-	{
-	  fprintf (f, "<author>%s</author>", current_voice_info->author);
-	}
-      if (current_voice_info->date)
-	{
-	  fprintf (f, "<date>%s</date>", current_voice_info->date);
-	}
-      if (current_voice_info->manuscript)
-	{
-	  fprintf (f,
-		   "<manuscript>%s</manuscript>",
-		   current_voice_info->manuscript);
-	}
-      if (current_voice_info->reference)
-	{
-	  fprintf (f,
-		   "<reference>%s</reference>",
-		   current_voice_info->reference);
-	}
-      if (current_voice_info->storage_place)
-	{
-	  fprintf (f,
-		   "<storage-place>%s</storage-place>",
-		   current_voice_info->storage_place);
-	}
-      if (current_voice_info->translator)
-	{
-	  fprintf (f,
-		   "<translator>%s</translator>",
-		   current_voice_info->translator);
-	}
-      if (current_voice_info->translation_date)
-	{
-	  fprintf (f,
-		   "<translation-date>%s</translation-date>",
-		   current_voice_info->translation_date);
 	}
 
       if (current_voice_info->style)
